@@ -17,14 +17,14 @@ use Carp qw(croak);
 use DateTime ();
 use DateTime::HiRes ();
 use DateTime::TimeZone ();
-use List::MoreUtils qw(all any none);
+use List::Util 1.33 qw(all any none);
 use Params::Validate ':all';
 use Scalar::Util qw(blessed);
 use Storable qw(dclone);
 
 use DateTime::Format::Natural::Utils qw(trim);
 
-our $VERSION = '1.18';
+our $VERSION = '1.24';
 
 validation_options(
     on_fail => sub
@@ -218,17 +218,35 @@ sub parse_datetime
             $self->_advance_future('md');
         }
     }
-    elsif ($date_string =~ /^(\d{4}(?:-\d{2}){0,2})T(\d{2}(?::\d{2}){0,2})$/) {
-        my ($date, $time) = ($1, $2);
-
+    elsif ($date_string =~ /^(\d{4}(?:-\d{2}){0,2})T(\d{2}(?::\d{2}){0,2})(?:[.,](\d+))?(Z|[+-]\d{2}(?::?\d{2})?)?$/) {
+        my ($date, $time, $fractional, $tz) = ($1, $2, $3, $4);
         my %args;
 
+        if (defined $tz) {
+            if ($tz eq 'Z') {
+                $self->{datetime}->set_time_zone('UTC');
+            } elsif ($tz =~ /^([+-])(\d{2})$/) {
+                $tz = "$1$2:00";
+            } else {
+                $tz =~ s/^([+-])(\d{2}):?(\d{2})$/$1$2:$3/;
+            }
+            $self->{datetime}->set_time_zone($tz);
+        }
+
         @args{qw(year month day)} = split /-/, $date;
         $args{$_} ||= 01 foreach qw(month day);
 
         @args{qw(hour minute second)} = split /:/, $time;
         $args{$_} ||= 00 foreach qw(minute second);
 
+        if (defined $fractional) {
+            my $nanosecond = $fractional;
+            if (length($nanosecond) < 9) {
+                $nanosecond .= '0' x (9 - length($nanosecond));
+            }
+            $args{nanosecond} = int($nanosecond);
+        }
+
         my $valid_date = $self->_check_date(map $args{$_}, qw(year month day));
         my $valid_time = $self->_check_time(map $args{$_}, qw(hour minute second));
 
@@ -240,9 +258,6 @@ sub parse_datetime
         }
 
         $self->_set(%args);
-
-        $self->{datetime}->truncate(to => 'second');
-        $self->_set_truncated;
         $self->_set_valid_exp;
     }
     elsif ($date_string =~ /^([+-]) (\d+?) ([a-zA-Z]+)$/x) {
@@ -1012,6 +1027,8 @@ valuable suggestions and patches:
  Jim Avera
  Olaf Alders
  Karen Etheridge
+ Graham Ollis
+ isla w
 
 =head1 SEE ALSO
 
